@@ -57,19 +57,30 @@ def detect(options, parser):
     return (0, None, {})
 
 def error(options, parser):
-    from atropos.error import ErrorEstimator, PairedErrorEstimator
+    from atropos.error import (
+        BaseQualityErrorEstimator, ShadowRegressionErrorEstimator,
+        PairedErrorEstimator)
     
-    batch_iterator, names, qualities = create_reader(options, parser, counter_magnitude="K")[0:3]
+    batch_iterator, names, qualities = create_reader(
+        options, parser, counter_magnitude="K")[0:3]
     try:
         if not qualities:
             parser.error("Cannot estimate error rate without base qualities")
         
+        if options.algorithm == 'quality':
+            estimator_class = BaseQualityErrorEstimator
+        elif options.algorithm == 'shadow':
+            estimator_class = ShadowRegressionErrorEstimator
+        
         if options.paired:
-            e = PairedErrorEstimator()
+            e = PairedErrorEstimator(
+                max_read_len=options.max_bases,
+                estimator_class=estimator_class)
         else:
-            e = ErrorEstimator()
+            e = estimator_class(max_read_len=options.max_bases)
         
         e.consume_all_batches(batch_iterator)
+    
     finally:
         batch_iterator.close()
     
