@@ -280,7 +280,7 @@ class Command(object):
         group.add_argument(
             "--batch-size",
             type=int_or_str, metavar="SIZE",
-            help="Number of records to process in each batch. (5000)")
+            help="Number of records to process in each batch. (1000)")
     
     def add_command_options(self):
         """
@@ -375,7 +375,7 @@ standard input/output. Without the -o option, output is sent to standard output.
         parser.set_defaults(
             zero_cap=None,
             action='trim',
-            batch_size=5000,
+            batch_size=None,
             known_adapter=None)
         
         group = parser.add_argument_group("Finding adapters",
@@ -1079,7 +1079,7 @@ standard input/output. Without the -o option, output is sent to standard output.
                 if not options.writer_process:
                     parser.error("Writer compression and --no-writer-process are mutually exclusive")
                 elif threads == 2:
-                    logging.getLogger.warn("Writer compression requires > 2 threads; using worker compression instead")
+                    logging.getLogger().warn("Writer compression requires > 2 threads; using worker compression instead")
                     options.compression = "worker"
             
             # Set queue sizes if necessary.
@@ -1094,6 +1094,17 @@ standard input/output. Without the -o option, output is sent to standard output.
                 options.result_queue_size = threads * (100 if options.compression == "worker" else 1000)
             elif options.result_queue_size > 0:
                 assert options.result_queue_size >= threads
+        
+            max_queue_size = options.read_queue_size + options.result_queue_size
+            if options.batch_size is None:
+                options.batch_size = max(1000, max_queue_size / 10e6)
+            elif options.batch_size * max_queue_size > 10e6:
+                logging.getLogger().warn("Combination of batch size {} and total queue size {} "
+                                         "may lead to excessive memory usage".format(
+                                         options.batch_size, max_queue_size))
+        
+        if options.batch_size is None:
+            options.batch_size = 1000
 
 COMMANDS['trim'] = TrimCommand
 
@@ -1167,20 +1178,34 @@ class ErrorCommand(Command):
     name = "error"
     usage = "atropos -se input.fastq error\natropos -pe in1.fq -pe2 in2.fq error"
     description = """
-Estimate the error rate from base qualities. This can help to
-determine the quality of your data, and to decide the value for
-the max error rate (-e) parameter. Normal error for an Illumina
-experiment is around 1% or less. We recommend setting -e to
-10 x the empirical error rate."""
+Estimate the sequencing error rate. This can help to determine
+the quality of your data, and to decide the value for the max
+error rate (-e) parameter. Normal error for an Illumina experiment
+is around 1% or less. We recommend setting -e to 10x the empirical
+error rate."""
     
     def add_command_options(self):
         parser = self.parser
         parser.set_defaults(max_reads=10000)
+        parser.add_argument(
+            "-a",
+            "--algorithm",
+            choices=('quality', 'shadow'), default="quality",
+            help="Method for estimating error rates; quality = base qualities, "
+                "shadow = shadow regression. Be advised that the 'shadow' "
+                "method is incredibly slow.")
+        parser.add_argument(
+            "-m",
+            "--max-bases",
+            type=int, default=None,
+            help="Maximum number of bases to use in the error calculation, "
+                "starting from the 5' end of the read.")
         parser.add_argument(
             "-o",
             "--output",
             type=writeable_file, default=STDOUT,
-            help="File in which to write the summary of the estimated error rates. (stdout)")
+            help="File in which to write the summary of the estimated error "
+                "rates. (stdout)")
 
 COMMANDS['error'] = ErrorCommand
 
